/*
  src2html - a tiny converter for DWARF2-XML xref
  Copyright (C) 2003, by gondow@jaist.ac.jp, All rights reserved.
  $Id: src2html.c,v 1.5 2003/06/24 16:05:19 gondow Exp $
 */
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>

#define DEFAULT_DIR	"./HTML"
static char *char_conv (int c);
static void src2html (char *filename);
static void make_dir (char *dirname);

/* ================================================================== */
static char *
char_conv (int c)
{
    switch (c) {
    case '<':  return "&lt;";
    case '>':  return "&gt;";
    case '&':  return "&amp;";
    default:   assert (0); return NULL;
    }
}

static void
make_dir (char *dirname)
{
    char buf [BUFSIZ];
    char *dir = strdup (dirname);
    char *p   = strrchr (dir, '/');
    *p = '\0';
    snprintf (buf, sizeof (buf), "mkdir -p %s", dir);
    system (buf);
    free (dir);
}

static void
src2html (char *filename)
{
    char buf [BUFSIZ];
    FILE *fp_in, *fp_out;
    int i, c, prev_c = '\n';

    snprintf (buf, sizeof (buf), "%s/%s.html", DEFAULT_DIR, filename);

    make_dir (buf);

    fp_in  = fopen (filename, "r");
    if (fp_in  == NULL) {
	fprintf (stderr, "file = %s\n", filename);
	perror ("fopen (r)");
	exit (1); 
    }
    fp_out = fopen (buf, "w");
    if (fp_out == NULL) {
	fprintf (stderr, "file = %s\n", buf);
	perror ("fopen (w)");
	exit (1);
    }

    fprintf (fp_out, "<html><body>\n<h1>%s</h1>"
	     "<a name=\"top\"/>\n<hr/>\n<pre>\n", filename);

    i = 1;
    while ((c = getc (fp_in)) != EOF) {
	if (prev_c == '\n') {
	    fprintf (fp_out, "<a name=\"%d\"><font color='blue'>%5d</font> ",
		     i, i);
	    i++;
	}
	if ((c == '<') || (c == '>') || (c == '&')) {
	    fprintf (fp_out, "%s", char_conv (c));
	} else {
	    putc (c, fp_out);
	}
	prev_c = c;
    }
    fprintf (fp_out, "</pre><hr/></body></html>\n");

    fclose (fp_in);
    fclose (fp_out);
}
/* ================================================================== */
int
main (int argc, char *argv [])
{
    int i;

    if (argc <= 1) {
	fprintf (stderr, "Usage: %s filename ...\n", argv [0]);
	exit (1);
    }

    for (i = 1; i < argc; i++) {
	src2html (argv [i]);
    }
}
/* ================================================================== */
